# Copyright (C) 2014-2025 Free Software Foundation, Inc.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import itertools

# This file is part of the GDB testsuite.  It tests Python-based
# frame-filters.
import gdb
from gdb.FrameDecorator import FrameDecorator


class FrameObjFile:
    def __init__(self):
        self.name = "Filter1"
        self.priority = 1
        self.enabled = False
        gdb.current_progspace().frame_filters["Progspace" + self.name] = self
        gdb.current_objfile().frame_filters["ObjectFile" + self.name] = self

    def filter(self, frame_iter):
        return frame_iter


class FrameObjFile2:
    def __init__(self):
        self.name = "Filter2"
        self.priority = 100
        self.enabled = True
        gdb.current_progspace().frame_filters["Progspace" + self.name] = self
        gdb.current_objfile().frame_filters["ObjectFile" + self.name] = self

    def filter(self, frame_iter):
        return frame_iter


FrameObjFile()
FrameObjFile2()



# Copyright (C) 2019-2025 Free Software Foundation, Inc.
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import gdb


class BadKey:
    def __repr__(self):
        return "Bad Key"


class ReallyBadKey:
    def __repr__(self):
        return BadKey()


class pycmd1(gdb.MICommand):
    def invoke(self, argv):
        if argv[0] == "int":
            return {"result": 42}
        elif argv[0] == "str":
            return {"result": "Hello world!"}
        elif argv[0] == "ary":
            return {"result": ["Hello", 42]}
        elif argv[0] == "dct":
            return {"result": {"hello": "world", "times": 42}}
        elif argv[0] == "bk1":
            return {"result": {BadKey(): "world"}}
        elif argv[0] == "bk2":
            return {"result": {1: "world"}}
        elif argv[0] == "bk3":
            return {"result": {ReallyBadKey(): "world"}}
        elif argv[0] == "tpl":
            return {"result": (42, "Hello")}
        elif argv[0] == "itr":
            return {"result": iter([1, 2, 3])}
        elif argv[0] == "nn1":
            return None
        elif argv[0] == "nn2":
            return {"result": [None]}
        elif argv[0] == "red":
            pycmd2("-pycmd")
            return None
        elif argv[0] == "nd1":
            return [1, 2, 3]
        elif argv[0] == "nd2":
            return 123
        elif argv[0] == "nd3":
            return "abc"
        elif argv[0] == "ik1":
            return {"xxx yyy": 123}
        elif argv[0] == "ik2":
            return {"result": {"xxx yyy": 123}}
        elif argv[0] == "ik3":
            return {"xxx+yyy": 123}
        elif argv[0] == "ik4":
            return {"xxx.yyy": 123}
        elif argv[0] == "ik5":
            return {"123xxxyyy": 123}
        elif argv[0] == "empty_key":
            return {"": 123}
        elif argv[0] == "dash-key":
            return {"the-key": 123}
        elif argv[0] == "exp":
            raise gdb.GdbError()
        else:
            raise gdb.GdbError("Invalid parameter: %s" % argv[0])


class pycmd2(gdb.MICommand):
    def invoke(self, argv):
        if argv[0] == "str":
            return {"result": "Ciao!"}
        elif argv[0] == "red":
            pycmd1("-pycmd")
            raise gdb.GdbError("Command redefined but we failing anyway")
        elif argv[0] == "new":
            pycmd1("-pycmd-new")
            return None
        else:
            raise gdb.GdbError("Invalid parameter: %s" % argv[0])


# This class creates a command that returns a string, which is passed
# when the command is created.
class pycmd3(gdb.MICommand):
    def __init__(self, name, msg, top_level):
        super(pycmd3, self).__init__(name)
        self._msg = msg
        self._top_level = top_level

    def invoke(self, args):
        return {self._top_level: {"msg": self._msg}}


# A command that is missing it's invoke method.
class no_invoke(gdb.MICommand):
    def __init__(self, name):
        super(no_invoke, self).__init__(name)


def free_invoke(obj, args):
    return {"result": args}


# Run some test involving catching exceptions.  It's easier to write
# these as a Python function which is then called from the exp script.
def run_exception_tests():
    print("PASS")


# Run some execute_mi tests.  This is easier to do from Python.
def run_execute_mi_tests():
    # Install the command.
    cmd = pycmd1("-pycmd")
    # Pass in a representative subset of the pycmd1 keys, and then
    # check that the result via MI is the same as the result via a
    # direct Python call.  Note that some results won't compare as
    # equal -- for example, a Python MI command can return a tuple,
    # but that will be translated to a Python list.
    for name in ("int", "str", "dct"):
        expect = cmd.invoke([name])
        got = gdb.execute_mi("-pycmd", name)
        if expect != got:
            print("FAIL: saw " + repr(got) + ", but expected " + repr(expect))
            return
    ok = False
    try:
        gdb.execute_mi("-pycmd", "exp")
    # Due to the "denaturation" problem, we have to expect a gdb.error
    # here and not a gdb.GdbError.
    except gdb.error:
        ok = True
    if not ok:
        print("FAIL: did not throw exception")
    print("PASS")



# Copyright (C) 2022-2025 Free Software Foundation, Inc.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


class Printer1:
    def to_string(self):
        n = gdb.parse_and_eval("called_from_pretty_printer ()")
        assert n == 23
        return "hahaha"


class Printer2:
    def to_string(self):
        n = gdb.parse_and_eval("called_from_pretty_printer ()")
        assert n == 23
        return "hohoho"


def lookup_function(val):
    if str(val.type) == "struct type_1":
        return Printer1()

    if str(val.type) == "struct type_2":
        return Printer2()

    return None


gdb.pretty_printers.append(lookup_function)



# Copyright (C) 2020-2025 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# A TUI window implemented in Python.

import gdb

the_window = None


tui_enabled = False


class TestWindow:
    def __init__(self, win):
        global the_window
        the_window = self
        self.count = 0
        self.win = win
        win.title = "This Is The Title"

    def render(self):
        self.win.erase()
        w = self.win.width
        h = self.win.height
        self.win.write(
            string="Test: " + str(self.count) + " " + str(w) + "x" + str(h),
            full_window=False,
        )
        self.count = self.count + 1

    # Tries to delete the title attribute.  GDB will throw an error.
    def remove_title(self):
        del self.win.title


gdb.register_window_type("test", TestWindow)


# Call REMOVE_TITLE on the global window object.
def delete_window_title():
    the_window.remove_title()


# A TUI window "constructor" that always fails.
def failwin(win):
    raise RuntimeError("Whoops")


# Change the title of the window.
def change_window_title():
    the_window.win.title = "New Title"


gdb.register_window_type("fail", failwin)


def set_tui_enabled(ev):
    global tui_enabled
    tui_enabled = ev.enabled


gdb.events.tui_enabled.connect(set_tui_enabled)



# Copyright 2022-2025 Free Software Foundation, Inc.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import gdb

threadOneExit = ""
threadTwoExit = ""
# we don't want to overwrite the 2nd thread's exit event, thus
# store it here. we don't care about it though.
mainThreadExit = ""


def thread_exited_handler(event):
    global threadOneExit, threadTwoExit, mainThreadExit
    print("{}".format(event))
    assert isinstance(event, gdb.ThreadExitedEvent)
    # Also check the inheritance.
    assert isinstance(event, gdb.ThreadEvent)
    if threadOneExit == "":
        threadOneExit = "event type: thread-exited. global num: {}".format(
            event.inferior_thread.global_num
        )
    else:
        if threadTwoExit == "":
            threadTwoExit = "event type: thread-exited. global num: {}".format(
                event.inferior_thread.global_num
            )
        else:
            mainThreadExit = "event type: thread-exited. global num: {}".format(
                event.inferior_thread.global_num
            )


class test_events(gdb.Command):
    """Test events."""

    def __init__(self):
        gdb.Command.__init__(self, "test-events", gdb.COMMAND_STACK)

    def invoke(self, arg, from_tty):
        gdb.events.thread_exited.connect(thread_exited_handler)
        print("Event testers registered.")


test_events()



# Copyright (C) 2025 Free Software Foundation, Inc.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys

# Avoid generating
# src/gdb/testsuite/gdb.python/__pycache__/gdb_leak_detector.cpython-<n>.pyc.
sys.dont_write_bytecode = True

import gdb_leak_detector


class color_leak_detector(gdb_leak_detector.gdb_leak_detector):
    def __init__(self):
        super().__init__(__file__)
        self.color = None

    def allocate(self):
        self.color = gdb.Color("red")

    def deallocate(self):
        self.color = None


color_leak_detector().run()
